/*----------------------------------------------------------------------------*/
/* ffscanf.c -- routine for scanning input from keyboard or file.  Accepts    */
/*              optional field name designator, and, if one is specified in   */
/*              input buffer, checks for agreement with the expected field.   */
/*              Also accepts comments in input line in standard C format, but */
/*              only when the comment is the only thing on the input line     */
/*              (i.e., no trailing comments).                                 */
/*              If field name designator is given and does not match the one  */
/*              expected, user is warned and program exits (this would        */
/*              usually occur only when reading from a file, as user wouldn't */
/*              enter designator from keyboard).  Otherwise, function returns */
/*              0 for FAIL, 1 for SUCCESS, or 2 for SKIP (parameter field was */
/*              blank (usually meaning that a default value is to be          */
/*              accepted).  In that case, the buffer passed in is unchanged.  */
/* NOTE:  Because variable numbers of arguments are not handled uniformly     */
/* across all compilers, 3 fixed-number-of-argument routines are given below, */
/* called ffscanf, ffscanf2, and ffscanf4, accepting 1, 2, and 4 arguments,   */
/* respectively.                                                              */
/*----------------------------------------------------------------------------*/
#include "stdafx.h"
#include "MSetar.h"
#include "MSetarDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "external.h"
 
/* typedef unsigned int BOOL; Declared in sgafunc.h */

#ifndef TRUE
  #define TRUE    1
  #define FALSE   0
#endif

#define FAIL    0
#define SUCC    1
#define SKIP    2

static char acLastBuffer[200];
static BOOL fIfLastBufferValid = FALSE;
static char acTmpBuffer[200];



// Prototipi di funzione
#ifdef PROTOTYPES_ACCEPTED
static void
	SeparateFieldName( char * Buffer,char ** fptr,char ** inptr );
#else
static void
	SeparateFieldName( /* char * Buffer,char ** fptr,char ** inptr */ );
#endif


#ifdef PROTOTYPES_ACCEPTED
int ffscanf(char *fieldname, FILE *fp, char *format, void *buffer)
#else

/* For non-ANSI versions of C-Compiler */
int ffscanf(fieldname, fp, format, buffer)
char   *fieldname;
FILE   *fp;
char   *format;
void   *buffer;

#endif
{

    int     cnt;
    char   *fieldnameptr;
    char   *inbufptr;
    int     len;
    int     commentflag = 0;


    if (fIfLastBufferValid == FALSE) {
		do {

			if (fgets(acLastBuffer, 199, fp) == NULL) {
				if (commentflag == 1) fprintf(stderr, "\nError in ffscanf:comments not correct.");
				return FAIL;
			}
			acLastBuffer[199] = '\0';
	        len = strlen(acLastBuffer);
	        if (acLastBuffer[len - 1] == '\n') acLastBuffer[len - 1] = '\0';

			switch (commentflag) {

				case 0:
					if (strstr(acLastBuffer, "/*") != NULL) {
						commentflag = 1;
						if (strstr(acLastBuffer, "*/") != NULL) commentflag = 2;
					}
					break;

				case 1:
					if (strstr(acLastBuffer, "*/") != NULL) commentflag = 2;
					break;

				case 2:
					if (strstr(acLastBuffer, "/*") != NULL) {
						commentflag = 1;
						if (strstr(acLastBuffer, "*/") != NULL) commentflag = 2;
					}
					else
						commentflag = 0;
					break;
			}
		} while (commentflag != 0);
	}// fIfLastBufferValid == FALSE
	
    
    strcpy(acTmpBuffer, acLastBuffer);
    fIfLastBufferValid = FALSE;

    SeparateFieldName(acTmpBuffer, &fieldnameptr, &inbufptr);

    if (strlen(fieldnameptr) != 0) {
		if (strcmp(fieldname, fieldnameptr)) {
			fprintf(stderr, "Field name %s is incorrect in this position (%s expected.)\n", fieldnameptr, fieldname);
			fflush(stderr);
			exit(1);
		}
    }

	/* now either it does not have a fieldname or the fieldname matches. */
    if (strlen(inbufptr) == 0) 	return SKIP;

	cnt = sscanf(inbufptr, format, buffer);

    if (!cnt) {
		fprintf(stderr, "\nInput error in %s:%s\n", fieldname, inbufptr);
		return FAIL;
    }
    return SUCC;
}



#ifdef PROTOTYPES_ACCEPTED
int ffscanf2(char *fieldname, FILE *fp, char *format, void *buffer1, void *buffer2)
#else

/* For old version of C-Compiler */
int
ffscanf2(fieldname, fp, format, buffer1, buffer2)
char   *fieldname;
FILE   *fp;
char   *format;
void *buffer1;
void *buffer2;
#endif

{

    int     cnt;
    char   *fieldnameptr;
    char   *inbufptr;
    int     len;
    int     commentflag = 0;

    if (fIfLastBufferValid == FALSE) {

		do {

			if (fgets(acLastBuffer, 199, fp) == NULL) {
				if (commentflag == 1) fprintf(stderr, "\nError in ffscanf:comments not correct.");
		    return FAIL;
			}

			acLastBuffer[199] = '\0';
	        len = strlen(acLastBuffer);
	        if (acLastBuffer[len - 1] == '\n') acLastBuffer[len - 1] = '\0';

			switch (commentflag) {
				case 0:
					if (strstr(acLastBuffer, "/*") != NULL) {
						commentflag = 1;
		                if (strstr(acLastBuffer, "*/") != NULL) commentflag = 2;
					}
					break;
				
				case 1:
					if (strstr(acLastBuffer, "*/") != NULL) commentflag = 2;
					break;

				case 2:
					if (strstr(acLastBuffer, "/*") != NULL) {
						commentflag = 1;
						if (strstr(acLastBuffer, "*/") != NULL)
							commentflag = 2;
					}
					else
						commentflag = 0;
					break;
			} // close switch
		} while (commentflag != 0);
	} // close if fIfLastBufferValid == FALSE


    strcpy(acTmpBuffer, acLastBuffer);
    fIfLastBufferValid = FALSE;

    SeparateFieldName(acTmpBuffer, &fieldnameptr, &inbufptr);

    if (strlen(fieldnameptr) != 0) {
		if (strcmp(fieldname, fieldnameptr)) {
			fprintf(stderr, "Field name %s is incorrect in this position (%s expected.)\n", fieldnameptr, fieldname);
	        exit(1);
		}
    }

    /* Now either no fieldname given or the fieldname matches name expected. */
    if (strlen(inbufptr) == 0) 	return SKIP;

    cnt = sscanf(inbufptr, format, buffer1, buffer2);

    if (!cnt) {
		fprintf(stderr, "\nInput error in %s:%s\n", fieldname, inbufptr);
	    return FAIL;
    }
    return SUCC;
}

#ifdef PROTOTYPES_ACCEPTED
int ffscanf4(char *fieldname, FILE *fp, char *format, void *buffer1
	     , void *buffer2, void *buffer3, void *buffer4)
#else

/* For old version of C-Compiler */
int
ffscanf4(fieldname, fp, format, buffer1, buffer2, buffer3, buffer4)
char   *fieldname;
FILE   *fp;
char   *format;
void *buffer1;
void *buffer2;
void *buffer3;
void *buffer4;
#endif

{
    int     cnt;
    char   *fieldnameptr;
    char   *inbufptr;
    int     len;
    int     commentflag = 0;

    if (fIfLastBufferValid == FALSE) {

		do {
			if (fgets(acLastBuffer, 199, fp) == NULL) {
				if (commentflag == 1) fprintf(stderr, "\nError in ffscanf:comments not correct.");
				return FAIL;
			}

			acLastBuffer[199] = '\0';
	        len = strlen(acLastBuffer);
	        if (acLastBuffer[len - 1] == '\n') acLastBuffer[len - 1] = '\0';

			switch (commentflag) {
				case 0:
					if (strstr(acLastBuffer, "/*") != NULL) {
						commentflag = 1;
		                if (strstr(acLastBuffer, "*/") != NULL) commentflag = 2;
					}
					break;

				case 1:
					if (strstr(acLastBuffer, "*/") != NULL) commentflag = 2;
					break;

				case 2:
					if (strstr(acLastBuffer, "/*") != NULL) {
						commentflag = 1;
		                if (strstr(acLastBuffer, "*/") != NULL) commentflag = 2;
					}
					else
						commentflag = 0;
					break;
			}//clos switch
		} while (commentflag != 0);
	}

	strcpy(acTmpBuffer, acLastBuffer);
    fIfLastBufferValid = FALSE;

    SeparateFieldName(acTmpBuffer, &fieldnameptr, &inbufptr);

    if (strlen(fieldnameptr) != 0) {
		if (strcmp(fieldname, fieldnameptr)) {
			fprintf(stderr, "Field name %s is incorrect in this position (%s expected.)\n", fieldnameptr, fieldname);
	        exit(1);
		}
    }
    
	/* Now either no fieldname given or the fieldname matches name expected. */
    if (strlen(inbufptr) == 0) 	return SKIP;


    cnt = sscanf(inbufptr, format, buffer1, buffer2, buffer3, buffer4);

    if (!cnt) {
		fprintf(stderr, "\nInput error in %s:%s\n", fieldname, inbufptr);
	    return FAIL;
    }
    
	return SUCC;
}





/* Tool function to separate fieldname and field value with '=' character. */
/* It uses the private buffer for output of the result. So fptr and inptr  */
/* are pointer's pointer to get the address of the private buffer.         */

static void SeparateFieldName(char   *Buffer, char  **fptr, char  **inptr) {

    static char fieldBuf[100];
    static char inBuf[100];
    int     fieldCnt = 0;
    int     i, k, inCnt = 0;

    int     len = strlen(Buffer);

    if (len >= 200) {
		Buffer[0] = '\0';
	    *fptr = Buffer;
	    *inptr = Buffer;
	    return;
    }

    fieldBuf[0] = '\0';
    inBuf[0] = '\0';
    fieldCnt = 0;
    inCnt = 0;

    for (i = 0; i < len; i++)
		if (Buffer[i] == '=')
			break;
    if (i < len) {
		/* now i indicates where the '=' occurs. */
	    for (k = 0; k < i; k++)
			if (Buffer[k] != ' ' && Buffer[k] != '\t')
				break;
		for (; k < i; k++)
			if (Buffer[k] != ' ' && Buffer[k] != '\t' && Buffer[k] != '0')
				fieldBuf[fieldCnt++] = Buffer[k];
		fieldBuf[fieldCnt] = '\0';
    }

    if (i >= len) k = 0;
    else          k = i + 1;

    /* now k indicates where the inbuf begins. */
    for (; k < len; k++)
		if (Buffer[k] != ' ' && Buffer[k] != '\t')
			break;
	for (; k < len; k++) {
		if ( /* Buffer[k]!=' ' && */ Buffer[k] != '\t' && Buffer[k] != '\0')
			inBuf[inCnt++] = Buffer[k];
		else if (Buffer[k] == '\t')
			inBuf[inCnt++] = ' ';
    }

    inBuf[inCnt] = '\0';

    *fptr = fieldBuf;
    *inptr = inBuf;
}

