/*----------------------------------------------------------------------------*/
/* utility.c - utility routines; contains copyright, repchar, skip,       */
/*             ithruj2int, int2ithruj, putfield, getfield, powerof2,      */
/*             ceilinglogbase2, floorlogbase2, chromtointarray,           */
/*             intarraytochrom, hamming, indextofloat, floattoindex,      */
/*             chromtointarray_uneq_flds, getfield_uneq_flds,       */
/*             putfield_uneq_flds, map_chromosome_to_std               */
/*             map_chromosome_to_std_uneq_flds,                        */
/*             CombineFileName, Lock, Unlock, MakeLockable, and KillTime. */
/* (routines using other patterns added, others modified to use local maps*/
/* with diffeent field sizes, 5/96, EDG */
/*----------------------------------------------------------------------------*/
#include "stdafx.h"
#include "MSetar.h"
#include "MSetarDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdio.h>
#include "external.h"

#ifndef SEEK_SET
#define SEEK_SET        0       /* Set file pointer to "offset" */
#endif

#ifndef SEEK_CUR
#define SEEK_CUR        1       /* Set file pointer to current plus "offset" */
#endif

#ifndef SEEK_END
#define SEEK_END        2       /* Set file pointer to EOF plus "offset" */
#endif

static int  variabledelay = 0;



void copyright(int parallel) {

	int i, iskip;
    int     ll = 60;
  
    iskip = (LINELENGTH - ll) / 2;
    skip(outfp, 1);
    repchar(outfp, " ", iskip);
    repchar(outfp, "*", ll);
    skip(outfp, 1);
    repchar(outfp, " ", iskip);
    fprintf(outfp, "Genetic ALgorithm Optimized for Portability and Parallelism System\n");
    repchar(outfp, " ", iskip);

    if (parallel)
		fprintf(outfp, "        Manypops -- Multiple Subpopulation Version        \n");
    else
        fprintf(outfp, "        Onepop   -- Single Population Version             \n");
  
    repchar(outfp, " ", iskip);
	fprintf(outfp, "               Ca' Foscari University                  \n");
    repchar(outfp, " ", iskip);
    fprintf(outfp, "                 Cucina Domenico                       \n");

    repchar(outfp, " ", iskip);
    repchar(outfp, "*", ll);
    skip(outfp, 2);
}

// Repeatedly write a character to stdout 
void repchar(FILE *outfp, char *ch, int repcount) {

	int     j;
    for (j = 1; j <= repcount; j++) fprintf(outfp, "%s", ch);
}

// Salta "skipcount" linee 
void skip(FILE *outfp, int skipcount) {

    int     j;
    for (j = 1; j <= skipcount; j++) fprintf(outfp, "\n");
}


/* interpret bits i thru j of a individual as an integer      */
/* j MUST BE greater than or equal to i AND j-i < UINTSIZE-1  */
/* from is a chromosome, represented as an array of unsigneds */
/* ***CAUTION*** This routine numbers bits from 1 up, NOT from */
/* 0 up, although it uses the entire word to pack the information */

int ithruj2int(int i, int j, unsigned int *from) {

	unsigned mask, temp;
    int     bound_flag;
    int     iisin, jisin;
    int     i1, j1, out;

    // 1 possibile errore
    if(j < i) {
		fprintf(stderr, "Error in ithruj2int: j < i\n");
	    exit(-1);
    }
    
	// 2 possibile errore
    if(j - i + 1 > UINTSIZE) {
	    fprintf(stderr, "Error in ithruj2int: j-i+1 > UINTSIZE\n");
	    exit(-1);
    }

    iisin = (i - 1) / UINTSIZE;
    jisin = (j - 1) / UINTSIZE;

    i1 = i - (iisin * UINTSIZE);
    j1 = j - (jisin * UINTSIZE);

    /* check if bits fall across a word boundary */
    if (iisin == jisin)
		bound_flag = 0;
    else
		bound_flag = 1;

    if (bound_flag == 0) {
	    mask = 1;
	    mask = (mask << (j1 - i1 + 1)) - 1;
	    mask = mask << (i1 - 1);
	    out = (from[iisin] & mask) >> (i1 - 1);
	    return (out);
    }
    else {
	    mask = 1;
	    mask = (mask << j1) - 1;
	    temp = from[jisin] & mask;
	    mask = 1;
	    mask = (mask << (UINTSIZE - i1 + 1)) - 1;
	    mask = mask << (i1 - 1);
	    out = ((from[iisin] & mask) >> (i1 - 1)) | temp << (UINTSIZE - i1 + 1);
	    return (out);
    }
}

/* Store a positive int value (no more than fieldlength bits) into chrom     */
/* in bits i through j, BUT ***CAUTION***:  This routine uses start and stop */
/* numbering from 1 - lchrom, ***NOT*** 0 - lchrom-1.  Use caution in        */
/* calling.  It stores 16 or 32 bits per word, as usual, however.            */

void int2ithruj(int i, int j, int valuearg, unsigned *chrom) {

	unsigned mask;
    int     iisin, jisin;
    int     i1, j1, value;

    value = valuearg;   
    if (j < i) {
	    fprintf(stderr, "Error in ithruj2int: j < i\n");
	    exit(-1);
    }

    if (j - i + 1 > UINTSIZE) {
	    fprintf(stderr, "Error in ithruj2int: j-i+1 > UINTSIZE\n");
	    exit(-1);
    }

    /* find out what word or words will write into */
    iisin = (i - 1) / UINTSIZE;
    jisin = (j - 1) / UINTSIZE;

    i1 = i - (iisin * UINTSIZE);
    j1 = j - (jisin * UINTSIZE);

    if (iisin == jisin) {
	    /* whole field in one unsigned word, prepare mask for use */
	    mask = 1;
	    mask = (mask << (j1 - i1 + 1)) - 1;
	    mask = mask << (i1 - 1);
	    /* use mask to clear field, 'or' shifted value into chrom */
	    chrom[iisin] &= (~mask);
	    chrom[iisin] |= (value << (i1 - 1));
    }
    else {
	    /* copy part of field into jisin, rest into iisin */
	    mask = 1;
	    mask = (mask << j1) - 1;
	    chrom[jisin] &= (~mask);
	    chrom[jisin] |= (value >> (UINTSIZE - i1 + 1));
	    mask = 1;
	    mask = (((mask << (UINTSIZE - i1 + 1)) - 1) << (i1 - 1));
	    chrom[iisin] &= (~mask);
	    chrom[iisin] |= (value << (i1 - 1));
    }
}


/* Fetches integer value from field n (numbered 0 - nbroffields-1) of a       */
/* chromosome.  (Works for binary, too.)  Modified 4/96 by EDG such that      */
/* each field may be of a different cardinality (and length), as specified in */
/* array field_sizes[], a global.   */
/* Last bit of field n (in [0, numfields-1]) is bit position field_ends[n],   */
/* (numbered 1,...,lchrom, as in Goldberg's SGA). */

int getfield(int n, int nbroffields, unsigned *chrom) {

	int     start;

    if (n >= nbroffields) {
	    fprintf(stderr, " Error in call to getfield, n (= %d) >= nbroffields (= %d)", n, nbroffields);
	    exit(-1);
    }
    if (n>0)
        start = field_ends[n-1] + 1;
    else
        start = 1;
    return ithruj2int(start, field_ends[n], chrom);
}



/* New 5/96, EDG, to allow using a different inversion pattern (e.g., one */
/* saved with a best individual) to decode an individual (i.e., one NOT */
/* using the CURRENT map_to_std as used by getfield(). */
/* NOTE:  DO NOT USE THIS UNLESS CURRENT MAP (and field_ends) DOES NOT */
/* MATCH THE LAYOUT OF THE INDIVIDUAL YOU ARE DECODING. */
/* Fetches integer value from field n (numbered 0 - nbroffields-1) of a       */
/* chromosome.  */
/* Last bit of field n (in [0, numfields-1]) is bit position field_ends[n],   */
/* (numbered 1,...,lchrom, as in Goldberg's SGA). */

int getfield_uneq_flds(int n, int nbroffields, unsigned *chrom, int *ends_of_fields) {

    int     start;

    if (n >= nbroffields) {
	    fprintf(stderr, " Error in call to getfield, n (= %d) >= nbroffields (= %d)", n, nbroffields);
	    exit(-1);
    }
    if (n>0)
        start = ends_of_fields[n-1] + 1;
    else
        start = 1;
    return ithruj2int(start, ends_of_fields[n], chrom);
}



/* Puts integer value into field n (numbered 0 - nbroffields-1) of a          */
/* chromosome.  (Works for binary, too.)  Each field may be of a different    */
/* cardinality (and length), as specified in array field_sizes[], a global.   */
/* Last bit of field n (in [0, numfields-1]) is bit position field_ends[n],   */
/* (numbered 1,...,lchrom, as in Goldberg's SGA). */

void putfield(int n, int nbroffields, int value, unsigned *chrom) {

    int     start;

    if (n >= nbroffields) {
	    fprintf(stderr, " Error in call to putfield, n (= %d) >= nbroffields (= %d)", n, nbroffields);
	    exit(-1);
    }
    if (n>0)
        start = field_ends[n-1] + 1;
    else
        start = 1;
    int2ithruj(start, field_ends[n], value, chrom);
}



/* Puts integer value into field n (numbered 0 - nbroffields-1) of a          */
/* chromosome, WHEN THAT CHROMOSOME DOES NOT NECESSARILY SHARE THE SAME */
/* map_to_std (i.e., has been permuted).  Parameter "ends_of_fields" is */
/* the array of field_ends which goes with the chrom passed in.  (Works */
/* for binary, too.)  DO NOT use this if chrom has the current pop's    */
/* inversion pattern, but rather for things like saved "bests" which may */
/* have been computed with a different pattern and preserved.  Each field */
/* may be of a different    */
/* cardinality (and length), as specified in array ends_of_fields[], passed */
/* in.  Last bit of field n (in [0, numfields-1]) is bit position         */
/* ends_of_fields[n], (numbered 1,...,lchrom, as in Goldberg's SGA). */

void putfield_uneq_flds(int n, int nbroffields, int value, unsigned *chrom, int *ends_of_fields) {

    int     start;

    if (n >= nbroffields) {
	    fprintf(stderr, " Error in call to putfield, n (= %d) >= nbroffields (= %d)", n, nbroffields);
	    exit(-1);
    }
    if (n>0)
        start = ends_of_fields[n-1] + 1;
    else
        start = 1;
    int2ithruj(start, ends_of_fields[n], value, chrom);
}



/* Returns 2**expon.                                                        */
long powerof2(int expon) {

    int     j;
    long    tmp;

    tmp = 1L;
    if (expon >= 0) {
	    for (j = 0; j < expon; j++) tmp *= 2;
	return tmp;
    }
    else
	    fprintf(stderr, " powerof2 is only for int returns (no neg. args).");
    exit(-1);
}

/* calculates ceiling(log base 2 of long argument),positive arguments      */
/* only (sign bit 0).                                                      */
int ceilinglogbase2(long longarg) {


    int     counter = 1;
    long    tmp;

    if (longarg <= 0) {
	    fprintf(stderr, "\n*** Call to ceilinglogbase2 with argument <= 0.\n");
	    exit(-1);
    }
    tmp = longarg;
    while ((tmp >>= 1) > 1) counter++;
    if (tmp << counter != longarg) counter++;
    return counter;
}


/* calculates the floor (log base 2 of integer argument), positive */
/* arguments only (sign bit 0).                                    */

int floorlogbase2(int integerarg) {

    int     counter = 1;

    if (integerarg <= 0) {
	    fprintf(stderr, "\n*** Call to floorlogbase2 with argument <= 0.\n");
	    exit(-1);
    }
    if (integerarg == 1) return 0;
    while ((integerarg >>= 1) > 1) counter++;
    return counter;

}



/* Inverse function to intarraytochrom();                   */
/* Loads intarray with the values from chromosome chrom.    */
/* Updated for variable-length fields by EDG, 5/96          */

void chromtointarray(int *intarray, unsigned *chrom) {

    int     i, start;

    start = 1;
    for (i = 0; i < numfields; i++) {
	    intarray[i] = ithruj2int(start, field_ends[i], chrom);
	    start = field_ends[i] + 1;
    }
}



/* Specialized inverse function to intarraytochrom(); BUT for use when the */                  
/* current map_to_std (and, hence, field_ends) does NOT MATCH the chrom */
/* being decoded (for example, for stored best individuals).  Added 5/96 */
/* by EDG. */
/* Loads intarray with the values from chromosome chrom, for any type of */
/* problem.                                         */
/* Updated for variable-length fields by EDG, 5/96 */

void chromtointarray_uneq_flds(int *intarray, unsigned *chrom, int *ends_of_fields) {

    int     i, start;

    start = 1;
    for (i = 0; i < numfields; i++) {
	    intarray[i] = ithruj2int(start, ends_of_fields[i], chrom);
	    start = ends_of_fields[i] + 1;
    }
}


/* Inverse function to chromtointarray();                                     */
/* Puts numfields (global) values from intarray into chromosome chrom.        */
/* Updated for variable-length fields by EDG, 5/96                            */

void intarraytochrom(int *intarray, unsigned *chrom) {

    int     i, start;

    start = 1;
    for (i = 0; i < numfields; i++) {
	    int2ithruj(start, field_ends[i], intarray[i], chrom);
	    start = field_ends[i] + 1;
    }
}

/* Given two chromosomes, calculates Hamming distance between them   */
int hamming(struct individual *critter1, struct individual *critter2) {


    int     j, k, stop, distance;
    unsigned matches, mask = 1;
    unsigned chrom1, chrom2;

    distance = 0;

    for (k = 0; k < chromsize; k++) {
	    chrom1 = critter1->chrom[k];
	    chrom2 = critter2->chrom[k];
	    matches = chrom1 ^ chrom2; /* 1's wherever chrom bits don't match */
	    /* now count the ones to count matches */
	    if (k == (chromsize - 1))
	        stop = lchrom - (k * UINTSIZE);
	    else
	        stop = UINTSIZE;

	    for (j = 0; j < stop; j++) {
	        if (matches & mask) distance++;
	    matches = matches >> 1;
		}
    }
    return distance;
}

/* scales a long int in range [0,maxlongint] into a float, linearly, in     */
/* range lower to upper (except if maxlongint ==0, returns 0., warns)       */
float indextofloat(long longint, long maxlongint, float lower, float upper) {

	if (maxlongint != 0) return lower + (upper - lower) * (longint / (float) maxlongint);
    else {
		fprintf(stderr, "\n*** WARNING, BAD maxlongint in call to indextofloat ***\n");
	    return 0.;
    }
}


/* maps a float in range [lower, upper] to a long in range [0,maxlongint],  */
/* linearly  (for speed, no safety checks).  Truncates rather than rounding */
long floattoindex(float floatvalue, float lower, float upper, long maxlongint) {

	long    tmp;

	if (floatvalue <= upper && floatvalue >= lower && upper != lower) {
	    tmp = (floatvalue * 1.0000000001 / (upper - lower)) * (double) maxlongint;
	    return tmp;
    }
    else {
	    fprintf(stderr, " Error in call to floattoindex:  bad argument.");
	    exit(-1);
    }
}












